@@ -1,8 +1,8 @@
 /*****************************************************************************
  * intf_eject.h: CD/DVD-ROM ejection handling functions
  *****************************************************************************
- * Copyright (C) 2001, 2002 VideoLAN
- * $Id: intf_eject.h,v 1.1 2002/01/09 02:01:14 sam Exp $
+ * Copyright (C) 2001, 2002 the VideoLAN team
+ * $Id: intf_eject.h 11664 2005-07-09 06:17:09Z courmisch $
  *
  * Author: Julien Blache <jb@technologeek.org>
  *
@@ -10,7 +10,7 @@
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -21,8 +21,6 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  *****************************************************************************/
 
-#ifndef PLUGIN
-int intf_Eject( const char * );
-#else
-#   define intf_Eject p_symbols->intf_Eject
-#endif
+#define intf_Eject(a,b) __intf_Eject(VLC_OBJECT(a),b)
+VLC_EXPORT( int, __intf_Eject, ( vlc_object_t *, const char * ) );
+
