@@ -1,8 +1,8 @@
 /*****************************************************************************
  * modules_inner.h : Macros used from within a module.
  *****************************************************************************
- * Copyright (C) 2001 VideoLAN
- * $Id: modules_inner.h,v 1.43 2004/01/09 12:23:47 gbazin Exp $
+ * Copyright (C) 2001 the VideoLAN team
+ * $Id: modules_inner.h 11664 2005-07-09 06:17:09Z courmisch $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -78,6 +78,12 @@
 #   define EXTERN_SYMBOL
 #endif
 
+#if defined( USE_DLL )
+#   define IMPORT_SYMBOL __declspec(dllimport)
+#else
+#   define IMPORT_SYMBOL
+#endif
+
 #define MODULE_STRING STRINGIFY( MODULE_NAME )
 
 /*
@@ -99,6 +105,7 @@
         p_module->b_unloadable = VLC_TRUE;                                    \
         p_module->b_reentrant = VLC_TRUE;                                     \
         p_module->psz_object_name = MODULE_STRING;                            \
+        p_module->psz_shortname = NULL;                                       \
         p_module->psz_longname = MODULE_STRING;                               \
         p_module->pp_shortcuts[ 0 ] = MODULE_STRING;                          \
         p_module->i_cpu = 0;                                                  \
@@ -142,6 +149,7 @@
                                 p_module->pp_shortcuts[ i_shortcut ];         \
     }                                                                         \
     p_submodule->psz_object_name = p_module->psz_object_name;                 \
+    p_submodule->psz_shortname = p_module->psz_shortname;                     \
     p_submodule->psz_longname = p_module->psz_longname;                       \
     p_submodule->psz_program = p_module->psz_program;                         \
     p_submodule->psz_capability = p_module->psz_capability;                   \
@@ -157,6 +165,9 @@
     p_submodule->pp_shortcuts[ i_shortcut ] = shortcut;                       \
     i_shortcut++
 
+#define set_shortname( desc )                                                 \
+    p_submodule->psz_shortname = desc
+
 #define set_description( desc )                                               \
     p_submodule->psz_longname = desc
 
