@@ -1,8 +1,8 @@
 /*****************************************************************************
  * decoder.h: header for vlc decoders
  *****************************************************************************
- * Copyright (C) 2002 VideoLAN
- * $Id: decoder.h,v 1.4 2004/01/25 18:17:08 zorglub Exp $
+ * Copyright (C) 2002 the VideoLAN team
+ * $Id: decoder.h 11664 2005-07-09 06:17:09Z courmisch $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,8 +35,6 @@ extern "C" {
  * Required internal headers
  *****************************************************************************/
 #include "vlc_block.h"
-#include "stream_control.h"
-#include "input_ext-dec.h"
 #include "vlc_video.h"
 #include "audio_output.h"
 #include "vlc_codec.h"
