@@ -1,8 +1,8 @@
 /*****************************************************************************
  * file.c
  *****************************************************************************
- * Copyright (C) 2001, 2002 VideoLAN
- * $Id: file.c 10953 2005-05-10 11:39:32Z courmisch $
+ * Copyright (C) 2001, 2002 the VideoLAN team
+ * $Id: file.c 13454 2005-11-29 14:51:01Z dionoea $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Eric Petit <titer@videolan.org>
@@ -131,6 +131,9 @@ static int Open( vlc_object_t *p_this )
     }
     if( !strcmp( p_access->psz_name, "-" ) )
     {
+#if defined(WIN32)
+        setmode (STDOUT_FILENO, O_BINARY);
+#endif
         p_access->p_sys->i_handle = STDOUT_FILENO;
         msg_Dbg( p_access, "using stdout" );
     }
