@@ -1,8 +1,8 @@
 /*****************************************************************************
  * s8tofloat32.c : converter from signed 8 bits integer to float32.
  *****************************************************************************
- * Copyright (C) 2002 VideoLAN
- * $Id: s8tofloat32.c 6961 2004-03-05 17:34:23Z sam $
+ * Copyright (C) 2002 the VideoLAN team
+ * $Id: s8tofloat32.c 11664 2005-07-09 06:17:09Z courmisch $
  *
  * Authors: Gildas Bazin <gbazin@netcourrier.com>
  *
@@ -43,6 +43,8 @@ static void DoWork    ( aout_instance_t *, aout_filter_t *, aout_buffer_t *,
  * Module descriptor
  *****************************************************************************/
 vlc_module_begin();
+    set_category( CAT_AUDIO );
+    set_subcategory( SUBCAT_AUDIO_MISC );
     set_description( _("audio filter for s8->float32 conversion") );
     set_capability( "audio filter", 1 );
     set_callbacks( Create, NULL );
