@@ -1,8 +1,8 @@
 /*****************************************************************************
  * sdl.c : SDL audio output plugin for vlc
  *****************************************************************************
- * Copyright (C) 2000-2002 VideoLAN
- * $Id: sdl.c 6961 2004-03-05 17:34:23Z sam $
+ * Copyright (C) 2000-2002 the VideoLAN team
+ * $Id: sdl.c 11664 2005-07-09 06:17:09Z courmisch $
  *
  * Authors: Michel Kaempf <maxx@via.ecp.fr>
  *          Sam Hocevar <sam@zoy.org>
@@ -65,8 +65,11 @@ static void SDLCallback ( void *, byte_t *, int );
  * Module descriptor
  *****************************************************************************/
 vlc_module_begin();
+    set_shortname( "SDL" );
     set_description( _("Simple DirectMedia Layer audio output") );
     set_capability( "audio output", 40 );
+    set_category( CAT_AUDIO );
+    set_subcategory( SUBCAT_AUDIO_AOUT );
     add_shortcut( "sdl" );
     set_callbacks( Open, Close );
 vlc_module_end();
