@@ -1,8 +1,8 @@
 /*****************************************************************************
  * encoder.c: video and audio encoder using the ffmpeg library
  *****************************************************************************
- * Copyright (C) 1999-2004 VideoLAN
- * $Id: encoder.c 9156 2004-11-05 14:57:53Z gbazin $
+ * Copyright (C) 1999-2004 the VideoLAN team
+ * $Id: encoder.c 13067 2005-10-31 23:12:10Z gbazin $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Gildas Bazin <gbazin@videolan.org>
@@ -33,7 +33,7 @@
 #include <vlc/decoder.h>
 
 /* ffmpeg header */
-#define HAVE_MMX
+#define HAVE_MMX 1
 #ifdef HAVE_FFMPEG_AVCODEC_H
 #   include <ffmpeg/avcodec.h>
 #else
@@ -134,12 +134,14 @@ struct encoder_sys_t
     float      f_rc_buffer_aggressivity;
     vlc_bool_t b_pre_me;
     vlc_bool_t b_hurry_up;
-    vlc_bool_t b_interlace;
+    vlc_bool_t b_interlace, b_interlace_me;
     float      f_i_quant_factor;
     int        i_noise_reduction;
     vlc_bool_t b_mpeg4_matrix;
     vlc_bool_t b_trellis;
     int        i_quality; /* for VBR */
+    float      f_lumi_masking, f_dark_masking, f_p_masking, f_border_masking;
+    int        i_luma_elim, i_chroma_elim;
 
     /* Used to work around stupid timestamping behaviour in libavcodec */
     uint64_t i_framenum;
@@ -147,17 +149,28 @@ struct encoder_sys_t
 };
 
 static const char *ppsz_enc_options[] = {
-    "keyint", "bframes", "vt", "qmin", "qmax", "hq", "strict_rc",
+    "keyint", "bframes", "vt", "qmin", "qmax", "hq", "strict-rc",
     "rc-buffer-size", "rc-buffer-aggressivity", "pre-me", "hurry-up",
     "interlace", "i-quant-factor", "noise-reduction", "mpeg4-matrix",
-    "trellis", "qscale", "strict", NULL
+    "trellis", "qscale", "strict", "lumi-masking", "dark-masking",
+    "p-masking", "border-masking", "luma-elim-threshold",
+    "chroma-elim-threshold", NULL
 };
 
+static const uint16_t mpa_bitrate_tab[2][15] =
+{
+    {0, 32, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 384},
+    {0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160}
+};
+
+static const uint16_t mpa_freq_tab[6] =
+{ 44100, 48000, 32000, 22050, 24000, 16000 };
+
 /*****************************************************************************
  * OpenEncoder: probe the encoder
  *****************************************************************************/
-extern int16_t ff_mpeg4_default_intra_matrix[];
-extern int16_t ff_mpeg4_default_non_intra_matrix[];
+extern int16_t IMPORT_SYMBOL ff_mpeg4_default_intra_matrix[];
+extern int16_t IMPORT_SYMBOL ff_mpeg4_default_non_intra_matrix[];
 
 int E_(OpenEncoder)( vlc_object_t *p_this )
 {
@@ -168,6 +181,9 @@ int E_(OpenEncoder)( vlc_object_t *p_this )
     int i_codec_id, i_cat;
     char *psz_namecodec;
     vlc_value_t val;
+    vlc_value_t lockval;
+
+    var_Get( p_enc->p_libvlc, "avcodec", &lockval );
 
     if( !E_(GetFfmpegCodec)( p_enc->fmt_out.i_codec, &i_cat, &i_codec_id,
                              &psz_namecodec ) )
@@ -222,6 +238,8 @@ int E_(OpenEncoder)( vlc_object_t *p_this )
     p_sys->p_buffer = NULL;
 
     p_sys->p_context = p_context = avcodec_alloc_context();
+    p_context->debug = config_GetInt( p_enc, "ffmpeg-debug" );
+    p_context->opaque = (void *)p_this;
 
     /* Set CPU capabilities */
     p_context->dsp_mask = 0;
@@ -257,6 +275,9 @@ int E_(OpenEncoder)( vlc_object_t *p_this )
     var_Get( p_enc, ENC_CFG_PREFIX "interlace", &val );
     p_sys->b_interlace = val.b_bool;
 
+    var_Get( p_enc, ENC_CFG_PREFIX "interlace-me", &val );
+    p_sys->b_interlace_me = val.b_bool;
+
     var_Get( p_enc, ENC_CFG_PREFIX "pre-me", &val );
     p_sys->b_pre_me = val.b_bool;
 
@@ -313,6 +334,19 @@ int E_(OpenEncoder)( vlc_object_t *p_this )
     if( val.i_int < - 1 || val.i_int > 1 ) val.i_int = 0;
     p_context->strict_std_compliance = val.i_int;
 
+    var_Get( p_enc, ENC_CFG_PREFIX "lumi-masking", &val );
+    p_sys->f_lumi_masking = val.f_float;
+    var_Get( p_enc, ENC_CFG_PREFIX "dark-masking", &val );
+    p_sys->f_dark_masking = val.f_float;
+    var_Get( p_enc, ENC_CFG_PREFIX "p-masking", &val );
+    p_sys->f_p_masking = val.f_float;
+    var_Get( p_enc, ENC_CFG_PREFIX "border-masking", &val );
+    p_sys->f_border_masking = val.f_float;
+    var_Get( p_enc, ENC_CFG_PREFIX "luma-elim-threshold", &val );
+    p_sys->i_luma_elim = val.i_int;
+    var_Get( p_enc, ENC_CFG_PREFIX "chroma-elim-threshold", &val );
+    p_sys->i_chroma_elim = val.i_int;
+
     if( p_enc->fmt_in.i_cat == VIDEO_ES )
     {
         int i_aspect_num, i_aspect_den;
@@ -328,8 +362,13 @@ int E_(OpenEncoder)( vlc_object_t *p_this )
         p_context->width = p_enc->fmt_in.video.i_width;
         p_context->height = p_enc->fmt_in.video.i_height;
 
+#if LIBAVCODEC_BUILD >= 4754
+        p_context->time_base.num = p_enc->fmt_in.video.i_frame_rate_base;
+        p_context->time_base.den = p_enc->fmt_in.video.i_frame_rate;
+#else
         p_context->frame_rate = p_enc->fmt_in.video.i_frame_rate;
         p_context->frame_rate_base= p_enc->fmt_in.video.i_frame_rate_base;
+#endif
 
         /* Defaults from ffmpeg.c */
         p_context->qblur = 0.5;
@@ -339,6 +378,15 @@ int E_(OpenEncoder)( vlc_object_t *p_this )
         p_context->i_quant_offset = 0.0;
         p_context->i_quant_factor = -0.8;
 
+        p_context->lumi_masking = p_sys->f_lumi_masking;
+        p_context->dark_masking = p_sys->f_dark_masking;
+        p_context->p_masking = p_sys->f_p_masking;
+#if LIBAVCODEC_BUILD >= 4741
+        p_context->border_masking = p_sys->f_border_masking;
+#endif
+        p_context->luma_elim_threshold = p_sys->i_luma_elim;
+        p_context->chroma_elim_threshold = p_sys->i_chroma_elim;
+
         if( p_sys->i_key_int > 0 )
             p_context->gop_size = p_sys->i_key_int;
         p_context->max_b_frames =
@@ -361,11 +409,29 @@ int E_(OpenEncoder)( vlc_object_t *p_this )
         p_sys->p_buffer_out = malloc( AVCODEC_MAX_VIDEO_FRAME_SIZE );
 
         p_enc->fmt_in.i_codec = VLC_FOURCC('I','4','2','0');
+        p_context->pix_fmt = E_(GetFfmpegChroma)( p_enc->fmt_in.i_codec );
+#if LIBAVCODEC_BUILD >= 4714
+        if( p_codec->pix_fmts )
+        {
+            const enum PixelFormat *p = p_codec->pix_fmts;
+            for( ; *p != -1; p++ )
+            {
+                if( *p == p_context->pix_fmt ) break;
+            }
+            if( *p == -1 ) p_context->pix_fmt = p_codec->pix_fmts[0];
+            p_enc->fmt_in.i_codec = E_(GetVlcChroma)( p_context->pix_fmt );
+        }
+#else
+        p_enc->fmt_in.i_codec = E_(GetVlcChroma)( p_context->pix_fmt );
+#endif
 
         if ( p_sys->b_strict_rc )
         {
             p_context->rc_max_rate = p_enc->fmt_out.i_bitrate;
             p_context->rc_buffer_size = p_sys->i_rc_buffer_size;
+            /* This is from ffmpeg's ffmpeg.c : */
+            p_context->rc_initial_buffer_occupancy
+                = p_sys->i_rc_buffer_size * 3/4;
             p_context->rc_buffer_aggressivity = p_sys->f_rc_buffer_aggressivity;
         }
 
@@ -390,15 +456,29 @@ int E_(OpenEncoder)( vlc_object_t *p_this )
 
         if ( p_sys->b_interlace )
         {
-            p_context->flags |= CODEC_FLAG_INTERLACED_DCT;
+            if ( p_context->height <= 280 )
+            {
+                if ( p_context->height != 16 || p_context->width != 16 )
+                    msg_Warn( p_enc,
+                        "disabling interlaced video because height=%d <= 280",
+                        p_context->height );
+            }
+            else
+            {
+                p_context->flags |= CODEC_FLAG_INTERLACED_DCT;
 #if LIBAVCODEC_BUILD >= 4698
-            p_context->flags |= CODEC_FLAG_INTERLACED_ME;
+                if ( p_sys->b_interlace_me )
+                    p_context->flags |= CODEC_FLAG_INTERLACED_ME;
+            }
 #endif
         }
 
         if ( p_sys->b_trellis )
             p_context->flags |= CODEC_FLAG_TRELLIS_QUANT;
 
+        if ( p_sys->i_qmin > 0 && p_sys->i_qmin == p_sys->i_qmax )
+            p_context->flags |= CODEC_FLAG_QSCALE;
+
 #if LIBAVCODEC_BUILD >= 4702
         if ( p_enc->i_threads >= 1 )
             p_context->thread_count = p_enc->i_threads;
@@ -425,6 +505,10 @@ int E_(OpenEncoder)( vlc_object_t *p_this )
     }
     else if( p_enc->fmt_in.i_cat == AUDIO_ES )
     {
+        /* work around bug in libmp3lame encoding */
+        if( i_codec_id == CODEC_ID_MP3 && p_enc->fmt_in.audio.i_channels > 2 )
+            p_enc->fmt_in.audio.i_channels = 2;
+
         p_enc->fmt_in.i_codec  = AOUT_FMT_S16_NE;
         p_context->sample_rate = p_enc->fmt_in.audio.i_rate;
         p_context->channels    = p_enc->fmt_in.audio.i_channels;
@@ -445,19 +529,67 @@ int E_(OpenEncoder)( vlc_object_t *p_this )
     p_context->extradata = NULL;
     p_context->flags |= CODEC_FLAG_GLOBAL_HEADER;
 
+    vlc_mutex_lock( lockval.p_address );
     if( avcodec_open( p_context, p_codec ) )
     {
-        if( p_enc->fmt_in.i_cat == AUDIO_ES && p_context->channels > 2 )
+        vlc_mutex_unlock( lockval.p_address );
+        if( p_enc->fmt_in.i_cat == AUDIO_ES &&
+             (p_context->channels > 2 || i_codec_id == CODEC_ID_MP2
+               || i_codec_id == CODEC_ID_MP3) )
         {
-            p_context->channels = 2;
-            p_enc->fmt_in.audio.i_channels = 2; // FIXME
+            if( p_context->channels > 2 )
+            {
+                p_context->channels = 2;
+                p_enc->fmt_in.audio.i_channels = 2; // FIXME
+                msg_Warn( p_enc, "stereo mode selected (codec limitation)" );
+            }
+
+            if( i_codec_id == CODEC_ID_MP2 || i_codec_id == CODEC_ID_MP3 )
+            {
+                int i_frequency, i;
+
+                for ( i_frequency = 0; i_frequency < 6; i_frequency++ )
+                {
+                    if ( p_enc->fmt_out.audio.i_rate
+                            == mpa_freq_tab[i_frequency] )
+                        break;
+                }
+                if ( i_frequency == 6 )
+                {
+                    msg_Err( p_enc, "MPEG audio doesn't support frequency=%d",
+                             p_enc->fmt_out.audio.i_rate );
+                    free( p_sys );
+                    return VLC_EGENERIC;
+                }
+
+                for ( i = 1; i < 14; i++ )
+                {
+                    if ( p_enc->fmt_out.i_bitrate / 1000
+                          <= mpa_bitrate_tab[i_frequency / 3][i] )
+                        break;
+                }
+                if ( p_enc->fmt_out.i_bitrate / 1000
+                      != mpa_bitrate_tab[i_frequency / 3][i] )
+                {
+                    msg_Warn( p_enc,
+                              "MPEG audio doesn't support bitrate=%d, using %d",
+                              p_enc->fmt_out.i_bitrate,
+                              mpa_bitrate_tab[i_frequency / 3][i] * 1000 );
+                    p_enc->fmt_out.i_bitrate =
+                        mpa_bitrate_tab[i_frequency / 3][i] * 1000;
+                    p_context->bit_rate = p_enc->fmt_out.i_bitrate;
+                }
+            }
+
+            p_context->codec = NULL;
+            vlc_mutex_lock( lockval.p_address );
             if( avcodec_open( p_context, p_codec ) )
             {
+                vlc_mutex_unlock( lockval.p_address );
                 msg_Err( p_enc, "cannot open encoder" );
                 free( p_sys );
                 return VLC_EGENERIC;
             }
-            msg_Warn( p_enc, "stereo mode selected (codec limitation)" );
         }
         else
         {
@@ -466,9 +598,15 @@ int E_(OpenEncoder)( vlc_object_t *p_this )
             return VLC_EGENERIC;
         }
     }
+    vlc_mutex_unlock( lockval.p_address );
 
     p_enc->fmt_out.i_extra = p_context->extradata_size;
-    p_enc->fmt_out.p_extra = p_context->extradata;
+    if( p_enc->fmt_out.i_extra )
+    {
+        p_enc->fmt_out.p_extra = malloc( p_enc->fmt_out.i_extra );
+        memcpy( p_enc->fmt_out.p_extra, p_context->extradata,
+                p_enc->fmt_out.i_extra );
+    }
     p_context->flags &= ~CODEC_FLAG_GLOBAL_HEADER;
 
     if( p_enc->fmt_in.i_cat == AUDIO_ES )
@@ -608,7 +746,7 @@ static block_t *EncodeVideo( encoder_t *p_enc, picture_t *p_pict )
     /* Let ffmpeg select the frame type */
     frame.pict_type = 0;
 
-    frame.repeat_pict = 2 - p_pict->i_nb_fields;
+    frame.repeat_pict = p_pict->i_nb_fields - 2;
 
 #if LIBAVCODEC_BUILD >= 4685
     frame.interlaced_frame = !p_pict->b_progressive;
@@ -624,9 +762,9 @@ static block_t *EncodeVideo( encoder_t *p_enc, picture_t *p_pict )
     if( 1 )
 #endif
     {
-        frame.pts = p_pict->date ? p_pict->date : AV_NOPTS_VALUE;
+        frame.pts = p_pict->date ? p_pict->date : (signed int) AV_NOPTS_VALUE;
 
-        if ( p_sys->b_hurry_up && frame.pts != AV_NOPTS_VALUE )
+        if ( p_sys->b_hurry_up && frame.pts != (signed int) AV_NOPTS_VALUE )
         {
             mtime_t current_date = mdate();
 
@@ -672,7 +810,7 @@ static block_t *EncodeVideo( encoder_t *p_enc, picture_t *p_pict )
         frame.pts = AV_NOPTS_VALUE;
     }
 
-    if ( frame.pts != AV_NOPTS_VALUE && frame.pts != 0 )
+    if ( frame.pts != (signed int) AV_NOPTS_VALUE && frame.pts != 0 )
     {
         if ( p_sys->i_last_pts == frame.pts )
         {
@@ -724,7 +862,7 @@ static block_t *EncodeVideo( encoder_t *p_enc, picture_t *p_pict )
             /* No delay -> output pts == input pts */
             p_block->i_pts = p_block->i_dts = p_pict->date;
         }
-        else if( p_sys->p_context->coded_frame->pts != AV_NOPTS_VALUE &&
+        else if( p_sys->p_context->coded_frame->pts != (signed int) AV_NOPTS_VALUE &&
             p_sys->p_context->coded_frame->pts != 0 &&
             p_sys->i_buggy_pts_detect != p_sys->p_context->coded_frame->pts )
         {
@@ -877,6 +1015,9 @@ void E_(CloseEncoder)( vlc_object_t *p_this )
 {
     encoder_t *p_enc = (encoder_t *)p_this;
     encoder_sys_t *p_sys = p_enc->p_sys;
+    vlc_value_t lockval;
+
+    var_Get( p_enc->p_libvlc, "avcodec", &lockval );
 
 #if LIBAVCODEC_BUILD >= 4702
     if ( p_sys->b_inited && p_enc->i_threads >= 1 )
@@ -898,7 +1039,9 @@ void E_(CloseEncoder)( vlc_object_t *p_this )
     }
 #endif
 
+    vlc_mutex_lock( lockval.p_address );
     avcodec_close( p_sys->p_context );
+    vlc_mutex_unlock( lockval.p_address );
     av_free( p_sys->p_context );
 
     if( p_sys->p_buffer ) free( p_sys->p_buffer );
