@@ -1,8 +1,8 @@
 /*****************************************************************************
  * intf.h: MacOS X interface module
  *****************************************************************************
- * Copyright (C) 2002-2004 VideoLAN
- * $Id: intf.h 8575 2004-08-29 19:48:09Z hartman $
+ * Copyright (C) 2002-2005 the VideoLAN team
+ * $Id: intf.h 12756 2005-10-02 19:47:03Z fkuehne $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *          Christophe Massiot <massiot@via.ecp.fr>
@@ -45,6 +45,10 @@ unsigned int CocoaKeyToVLC( unichar i_key );
  * the translated string. the translation should be '1:translatedstring' though */
 #define _ANS(s) [[[VLCMain sharedInstance] localizedString: _(s)] substringFromIndex:2]
 
+#define MACOS_VERSION [[[NSDictionary dictionaryWithContentsOfFile: \
+            @"/System/Library/CoreServices/SystemVersion.plist"] \
+            objectForKey: @"ProductVersion"] floatValue]
+
 /*****************************************************************************
  * intf_sys_t: description and status of the interface
  *****************************************************************************/
@@ -63,8 +67,10 @@ struct intf_sys_t
     /* interface update */
     vlc_bool_t b_intf_update;
     vlc_bool_t b_playlist_update;
+    vlc_bool_t b_playmode_update;
     vlc_bool_t b_current_title_update;
     vlc_bool_t b_fullscreen_update;
+    vlc_bool_t b_volume_update;
 
     /* menus handlers */
     vlc_bool_t b_input_update;
@@ -82,8 +88,22 @@ struct intf_sys_t
 {
     intf_thread_t *p_intf;      /* The main intf object */
     id o_prefs;                 /* VLCPrefs       */
+    id o_about;                 /* VLAboutBox     */
+    id o_open;                  /* VLCOpen        */
+    id o_wizard;                /* VLCWizard      */
+    id o_extended;              /* VLCExtended    */
+    id o_bookmarks;             /* VLCBookmarks   */
+    /*id o_update;                 VLCUpdate      */
+    BOOL nib_main_loaded;       /* reference to the main-nib */
+    BOOL nib_open_loaded;       /* reference to the open-nib */
+    BOOL nib_about_loaded;      /* reference to the about-nib */
+    BOOL nib_wizard_loaded;     /* reference to the wizard-nib */
+    BOOL nib_extended_loaded;   /* reference to the extended-nib */
+    BOOL nib_bookmarks_loaded;  /* reference to the bookmarks-nib */ 
+    /*BOOL nib_update_loaded;      reference to the update-nib */
 
     IBOutlet id o_window;       /* main window    */
+    IBOutlet id o_playlist_view;/* playlist view  */
     IBOutlet id o_scrollfield;  /* info field     */
     IBOutlet id o_timefield;    /* time field     */
     IBOutlet id o_timeslider;   /* time slider    */
@@ -98,6 +118,7 @@ struct intf_sys_t
     IBOutlet id o_btn_ff;       /* btn fast forward     */
     IBOutlet id o_btn_next;     /* btn next       */
     IBOutlet id o_btn_fullscreen;/* btn fullscreen      */
+    IBOutlet id o_btn_playlist; /* btn playlist   */
 
     NSImage * o_img_play;       /* btn play img   */
     NSImage * o_img_pause;      /* btn pause img  */
@@ -128,6 +149,7 @@ struct intf_sys_t
 
     IBOutlet id o_mi_about;
     IBOutlet id o_mi_prefs;
+    /*IBOutlet id o_mi_checkForUpdate;*/
     IBOutlet id o_mi_add_intf;
     IBOutlet id o_mu_add_intf;
     IBOutlet id o_mi_services;
@@ -143,6 +165,7 @@ struct intf_sys_t
     IBOutlet id o_mi_open_net;
     IBOutlet id o_mi_open_recent;
     IBOutlet id o_mi_open_recent_cm;
+    IBOutlet id o_mi_open_wizard;
 
     IBOutlet id o_mu_edit;
     IBOutlet id o_mi_cut;
@@ -194,6 +217,7 @@ struct intf_sys_t
     IBOutlet id o_mi_fittoscreen;
     IBOutlet id o_mi_fullscreen;
     IBOutlet id o_mi_floatontop;
+    IBOutlet id o_mi_snapshot;
     IBOutlet id o_mi_videotrack;
     IBOutlet id o_mu_videotrack;
     IBOutlet id o_mi_screen;
@@ -210,6 +234,8 @@ struct intf_sys_t
     IBOutlet id o_mi_close_window;
     IBOutlet id o_mi_controller;
     IBOutlet id o_mi_equalizer;
+    IBOutlet id o_mi_extended;
+    IBOutlet id o_mi_bookmarks;
     IBOutlet id o_mi_playlist;
     IBOutlet id o_mi_info;
     IBOutlet id o_mi_messages;
@@ -221,6 +247,8 @@ struct intf_sys_t
     IBOutlet id o_mi_reportabug;
     IBOutlet id o_mi_website;
     IBOutlet id o_mi_license;
+    IBOutlet id o_mi_donation;
+    IBOutlet id o_mi_forum;
 
     /* dock menu */
     IBOutlet id o_dmi_play;
@@ -228,6 +256,14 @@ struct intf_sys_t
     IBOutlet id o_dmi_next;
     IBOutlet id o_dmi_previous;
     IBOutlet id o_dmi_mute;
+
+    bool b_small_window;
+
+    mtime_t i_end_scroll;
+
+    NSSize o_size_with_playlist;
+
+    int     i_lastShownVolume;
 }
 
 + (VLCMain *)sharedInstance;
@@ -238,6 +274,8 @@ struct intf_sys_t
 - (id)getControls;
 - (id)getPlaylist;
 - (id)getInfo;
+- (id)getWizard;
+- (id)getBookmarks;
 - (void)terminate;
 - (NSString *)localizedString:(char *)psz;
 - (char *)delocalizeString:(NSString *)psz;
@@ -249,6 +287,8 @@ struct intf_sys_t
 - (void)manage;
 - (void)manageIntf:(NSTimer *)o_timer;
 - (void)setupMenus;
+- (void)setScrollField:(NSString *)o_string stopAfter:(int )timeout;
+- (void)resetScrollField;
 
 - (void)updateMessageArray;
 - (void)playStatusUpdated:(int) i_status;
@@ -259,15 +299,31 @@ struct intf_sys_t
 - (IBAction)clearRecentItems:(id)sender;
 - (void)openRecentItem:(id)sender;
 
+- (IBAction)intfOpenFile:(id)sender;
+- (IBAction)intfOpenFileGeneric:(id)sender;
+- (IBAction)intfOpenDisc:(id)sender;
+- (IBAction)intfOpenNet:(id)sender;
+
+- (IBAction)showWizard:(id)sender;
+- (IBAction)showExtended:(id)sender;
+- (IBAction)showBookmarks:(id)sender;
+
+- (IBAction)viewAbout:(id)sender;
 - (IBAction)viewPreferences:(id)sender;
+/*- (IBAction)checkForUpdate:(id)sender;*/
 - (IBAction)closeError:(id)sender;
 - (IBAction)openReadMe:(id)sender;
 - (IBAction)openDocumentation:(id)sender;
 - (IBAction)reportABug:(id)sender;
 - (IBAction)openWebsite:(id)sender;
 - (IBAction)openLicense:(id)sender;
+- (IBAction)openForum:(id)sender;
+- (IBAction)openDonate:(id)sender;
 - (IBAction)openCrashLog:(id)sender;
 
+- (IBAction)togglePlaylist:(id)sender;
+- (void)updateTogglePlaylistState;
+
 - (void)windowDidBecomeKey:(NSNotification *)o_notification;
 
 @end
