@@ -1,8 +1,8 @@
 /*****************************************************************************
  * skin_common.hpp
  *****************************************************************************
- * Copyright (C) 2003 VideoLAN
- * $Id: skin_common.hpp 8495 2004-08-22 12:50:39Z asmax $
+ * Copyright (C) 2003 the VideoLAN team
+ * $Id: skin_common.hpp 12501 2005-09-09 19:20:34Z gbazin $
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
  *          Olivier Teulire <ipkiss@via.ecp.fr>
@@ -46,6 +46,13 @@ class ThemeRepository;
 #   define M_PI 3.14159265358979323846
 #endif
 
+#ifdef _MSC_VER
+// turn off 'warning C4355: 'this' : used in base member initializer list'
+#pragma warning ( disable:4355 )
+// turn off 'identifier was truncated to '255' characters in the debug info'
+#pragma warning ( disable:4786 )
+#endif
+
 // Useful macros
 #define SKINS_DELETE( p ) \
    if( p ) \
@@ -109,28 +116,6 @@ class SkinObject
         /// interface)
         intf_thread_t *getIntf() const { return m_pIntf; }
 
-        /// Class for callbacks
-        class Callback {
-            public:
-                /// Type for callback methods
-                typedef void (*CallbackFunc_t)( SkinObject* );
-
-                /// Create a callback with the given object and function
-                Callback( SkinObject *pObj, CallbackFunc_t pFunc ):
-                    m_pObj( pObj ), m_pFunc( pFunc ) {}
-                ~Callback() {}
-
-                /// Getters
-                SkinObject *getObj() const { return m_pObj; }
-                CallbackFunc_t getFunc() const { return m_pFunc; }
-
-            private:
-                /// Pointer on the callback object
-                SkinObject *const m_pObj;
-                /// Pointer on the callback method
-                CallbackFunc_t m_pFunc;
-        };
-
     private:
         intf_thread_t *m_pIntf;
 };
