@@ -1,8 +1,8 @@
 /*****************************************************************************
  * skin_main.cpp
  *****************************************************************************
- * Copyright (C) 2003 VideoLAN
- * $Id: skin_main.cpp 10681 2005-04-15 14:01:27Z gbazin $
+ * Copyright (C) 2003 the VideoLAN team
+ * $Id: skin_main.cpp 12034 2005-08-05 17:52:46Z massiot $
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
  *          Olivier Teulire <ipkiss@via.ecp.fr>
@@ -85,6 +85,7 @@ static int Open( vlc_object_t *p_this )
     if( p_intf->p_sys->p_playlist == NULL )
     {
         msg_Err( p_intf, "No playlist object found" );
+	msg_Unsubscribe( p_intf, p_intf->p_sys->p_sub );
         return VLC_EGENERIC;
     }
 
@@ -109,26 +110,36 @@ static int Open( vlc_object_t *p_this )
     if( OSFactory::instance( p_intf ) == NULL )
     {
         msg_Err( p_intf, "Cannot initialize OSFactory" );
+	vlc_object_release( p_intf->p_sys->p_playlist );
+	msg_Unsubscribe( p_intf, p_intf->p_sys->p_sub );
         return VLC_EGENERIC;
     }
     if( AsyncQueue::instance( p_intf ) == NULL )
     {
         msg_Err( p_intf, "Cannot initialize AsyncQueue" );
+	vlc_object_release( p_intf->p_sys->p_playlist );
+	msg_Unsubscribe( p_intf, p_intf->p_sys->p_sub );
         return VLC_EGENERIC;
     }
     if( Interpreter::instance( p_intf ) == NULL )
     {
         msg_Err( p_intf, "Cannot instanciate Interpreter" );
+	vlc_object_release( p_intf->p_sys->p_playlist );
+	msg_Unsubscribe( p_intf, p_intf->p_sys->p_sub );
         return VLC_EGENERIC;
     }
     if( VarManager::instance( p_intf ) == NULL )
     {
         msg_Err( p_intf, "Cannot instanciate VarManager" );
+	vlc_object_release( p_intf->p_sys->p_playlist );
+	msg_Unsubscribe( p_intf, p_intf->p_sys->p_sub );
         return VLC_EGENERIC;
     }
     if( VlcProc::instance( p_intf ) == NULL )
     {
         msg_Err( p_intf, "Cannot initialize VLCProc" );
+	vlc_object_release( p_intf->p_sys->p_playlist );
+	msg_Unsubscribe( p_intf, p_intf->p_sys->p_sub );
         return VLC_EGENERIC;
     }
     Dialogs::instance( p_intf );
@@ -162,7 +173,7 @@ static void Close( vlc_object_t *p_this )
         vlc_object_release( p_intf->p_sys->p_playlist );
     }
 
-   // Unsubscribe from messages bank
+    // Unsubscribe from messages bank
     msg_Unsubscribe( p_intf, p_intf->p_sys->p_sub );
 
     // Destroy structure
