@@ -1,8 +1,8 @@
 /*****************************************************************************
  * qte_main.c : QT Embedded wrapper for gte_main
  *****************************************************************************
- * Copyright (C) 2003 VideoLAN
- * $Id: qte_main.cpp,v 1.4 2003/03/06 10:11:33 jpsaman Exp $
+ * Copyright (C) 2003 the VideoLAN team
+ * $Id: qte_main.cpp 11664 2005-07-09 06:17:09Z courmisch $
  *
  * Authors: Jean-Paul Saman <jpsaman@wxs.nl>
  *
@@ -41,7 +41,8 @@ typedef struct qte_thread_t
     VLC_COMMON_MEMBERS
 
     QApplication*       p_qte_application;
-    QWidget* 		p_qte_widget;
+    QWidget*            p_qte_widget;
+    bool                b_gui_server;
 
 } qte_thread_t;
 
@@ -62,9 +63,15 @@ static qte_thread_t * p_qte_main = NULL;
 /*****************************************************************************
  * Module descriptor
  *****************************************************************************/
+#define STANDALONE_TEXT N_("Run as standalone Qt/Embedded GUI Server")
+#define STANDALONE_LONGTEXT N_("Use this option to run as standalone " \
+    "Qt/Embedded GUI Server. This option is equivalent to the -qws option " \
+    "from normal Qt.")
+
 vlc_module_begin();
-    set_description( _("Qt Embedded helper module") );
-    set_capability( "qte_main", 90 );
+    set_description( _("Qt Embedded GUI helper") );
+    set_capability( "gui-helper", 90 );
+    add_bool( "qte-guiserver", 0, NULL, STANDALONE_TEXT, STANDALONE_LONGTEXT, VLC_FALSE );
     add_shortcut( "qte" );
     set_callbacks( Open, Close );
 vlc_module_end();
@@ -94,8 +101,8 @@ static int Open( vlc_object_t *p_this )
 
     p_qte_main = (qte_thread_t *) vlc_object_create( p_this, sizeof(qte_thread_t) );
 
-    /* Launch the gtk_main() thread. It will not return until it has
-     * called gdk_threads_enter(), which ensures us thread safety. */
+    /* Launch the QApplication::exec() thread. It will not return until the
+     * application is properly initialized, which ensures us thread safety. */
     if( vlc_thread_create( p_qte_main, "qte_main", QteMain,
                            VLC_THREAD_PRIORITY_LOW, VLC_TRUE ) )
     {
@@ -109,6 +116,9 @@ static int Open( vlc_object_t *p_this )
     i_refcount++;
     vlc_mutex_unlock( (vlc_mutex_t *) lockval.p_address );
 
+    vlc_object_attach( p_qte_main, p_this );
+    msg_Dbg( p_this, "qte_main running" );
+
     return VLC_SUCCESS;
 }
 
@@ -136,6 +146,9 @@ static void Close( vlc_object_t *p_this )
     delete p_qte_main->p_qte_widget;
     delete p_qte_main->p_qte_application;
 
+    msg_Dbg( p_this, "Detaching qte_main" );
+    vlc_object_detach( p_qte_main );
+
     vlc_object_destroy( p_qte_main );
     p_qte_main = NULL;
 
@@ -151,15 +164,24 @@ static void Close( vlc_object_t *p_this )
  *****************************************************************************/
 static void QteMain( qte_thread_t *p_this )
 {
-    int argc = 0;
+    int i_argc = 1;
 
-    QApplication* pApp = new QApplication(argc, NULL);
+    p_this->b_gui_server = VLC_FALSE;
+    if( config_GetInt( p_this, "qte-guiserver" ) )
+    {
+        msg_Dbg( p_this, "Running as Qt Embedded standalone GuiServer" );
+        p_this->b_gui_server = VLC_TRUE;
+    }
+
+    /* Run as standalone GuiServer or as GuiClient. */
+    QApplication* pApp = new QApplication(i_argc, NULL,
+        (p_this->b_gui_server ? (QApplication::GuiServer):(QApplication::GuiClient)) );
     if(pApp)
     {
         p_this->p_qte_application = pApp;
     }
 
-    QWidget* pWidget = new QWidget();
+    QWidget* pWidget = new QWidget(0, _("video") );
     if(pWidget)
     {
         p_this->p_qte_widget = pWidget;
