@@ -1,8 +1,8 @@
 /*****************************************************************************
  * h264.c: h264/avc video packetizer
  *****************************************************************************
- * Copyright (C) 2001, 2002 VideoLAN
- * $Id: h264.c 9031 2004-10-22 11:11:27Z gbazin $
+ * Copyright (C) 2001, 2002 the VideoLAN team
+ * $Id: h264.c 12544 2005-09-13 22:01:20Z gbazin $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Eric Petit <titer@videolan.org>
@@ -42,6 +42,8 @@ static int  Open ( vlc_object_t * );
 static void Close( vlc_object_t * );
 
 vlc_module_begin();
+    set_category( CAT_SOUT );
+    set_subcategory( SUBCAT_SOUT_PACKETIZER );
     set_description( _("H264 video packetizer") );
     set_capability( "packetizer", 50 );
     set_callbacks( Open, Close );
@@ -65,14 +67,13 @@ struct decoder_sys_t
     vlc_bool_t b_slice;
     block_t    *p_frame;
 
-    int64_t      i_dts;
-    int64_t      i_pts;
-    unsigned int i_flags;
-
     vlc_bool_t   b_sps;
+    vlc_bool_t   b_pps;
 
     /* avcC data */
     int i_avcC_length_size;
+    block_t *p_sps;
+    block_t *p_pps;
 
     /* Useful values of the Sequence Parameter Set */
     int i_log2_max_frame_num;
@@ -101,7 +102,8 @@ enum nal_unit_type_e
     NAL_SLICE_IDR   = 5,    /* ref_idc != 0 */
     NAL_SEI         = 6,    /* ref_idc == 0 */
     NAL_SPS         = 7,
-    NAL_PPS         = 8
+    NAL_PPS         = 8,
+    NAL_AU_DELIMITER= 9
     /* ref_idc == 0 for 6,9,10,11,12 */
 };
 
@@ -150,10 +152,10 @@ static int Open( vlc_object_t *p_this )
     p_sys->bytestream = block_BytestreamInit( p_dec );
     p_sys->b_slice = VLC_FALSE;
     p_sys->p_frame = NULL;
-    p_sys->i_dts   = 0;
-    p_sys->i_pts   = 0;
-    p_sys->i_flags = 0;
     p_sys->b_sps   = VLC_FALSE;
+    p_sys->b_pps   = VLC_FALSE;
+    p_sys->p_sps   = 0;
+    p_sys->p_pps   = 0;
 
     p_sys->i_nal_type = -1;
     p_sys->i_nal_ref_idc = -1;
@@ -184,6 +186,8 @@ static int Open( vlc_object_t *p_this )
             int i_length = GetWBE( p );
             block_t *p_sps = nal_get_annexeb( p_dec, p + 2, i_length );
 
+            p_sys->p_sps = block_Duplicate( p_sps );
+            p_sps->i_pts = p_sps->i_dts = mdate();
             ParseNALBlock( p_dec, p_sps );
             p += 2 + i_length;
         }
@@ -194,6 +198,8 @@ static int Open( vlc_object_t *p_this )
             int i_length = GetWBE( p );
             block_t *p_pps = nal_get_annexeb( p_dec, p + 2, i_length );
 
+            p_sys->p_pps = block_Duplicate( p_pps );
+            p_pps->i_pts = p_pps->i_dts = mdate();
             ParseNALBlock( p_dec, p_pps );
             p += 2 + i_length;
         }
@@ -207,6 +213,22 @@ static int Open( vlc_object_t *p_this )
     {
         /* Set callback */
         p_dec->pf_packetize = Packetize;
+
+        /* */
+        if( p_dec->fmt_in.i_extra > 0 )
+        {
+            block_t *p_init = block_New( p_dec, p_dec->fmt_in.i_extra );
+            block_t *p_pic;
+
+            memcpy( p_init->p_buffer, p_dec->fmt_in.p_extra,
+                    p_dec->fmt_in.i_extra );
+
+            while( ( p_pic = Packetize( p_dec, &p_init ) ) )
+            {
+                /* Should not occur because we should only receive SPS/PPS */
+                block_Release( p_pic );
+            }
+        }
     }
 
     return VLC_SUCCESS;
@@ -220,6 +242,8 @@ static void Close( vlc_object_t *p_this )
     decoder_t *p_dec = (decoder_t*)p_this;
     decoder_sys_t *p_sys = p_dec->p_sys;
 
+    if( p_sys->p_sps ) block_Release( p_sys->p_sps );
+    if( p_sys->p_pps ) block_Release( p_sys->p_pps );
     block_BytestreamRelease( &p_sys->bytestream );
     free( p_sys );
 }
@@ -265,15 +289,10 @@ static block_t *Packetize( decoder_t *p_dec, block_t **pp_block )
             case STATE_NEXT_SYNC:
                 /* Find the next startcode */
                 if( block_FindStartcodeFromOffset( &p_sys->bytestream,
-                      &p_sys->i_offset, p_sys->startcode, 3 ) != VLC_SUCCESS)
+                      &p_sys->i_offset, p_sys->startcode+1, 3 ) != VLC_SUCCESS)
                 {
-                    if( block_FindStartcodeFromOffset( &p_sys->bytestream,
-                          &p_sys->i_offset, p_sys->startcode+1, 3 ) !=
-                        VLC_SUCCESS )
-                    {
-                        /* Need more data */
-                        return NULL;
-                    }
+                    /* Need more data */
+                    return NULL;
                 }
 
                 /* Get the new fragment and set the pts/dts */
@@ -284,6 +303,7 @@ static block_t *Packetize( decoder_t *p_dec, block_t **pp_block )
                 block_GetBytes( &p_sys->bytestream, p_pic->p_buffer,
                                 p_pic->i_buffer );
 
+                if( !p_pic->p_buffer[p_pic->i_buffer-1] ) p_pic->i_buffer--;
                 p_sys->i_offset = 0;
 
                 /* Parse the NAL */
@@ -292,6 +312,10 @@ static block_t *Packetize( decoder_t *p_dec, block_t **pp_block )
                     p_sys->i_state = STATE_NOSYNC;
                     break;
                 }
+#if 0
+                msg_Dbg( p_dec, "pts="I64Fd" dts="I64Fd,
+                         p_pic->i_pts, p_pic->i_dts );
+#endif
 
                 /* So p_block doesn't get re-added several times */
                 *pp_block = block_BytestreamPop( &p_sys->bytestream );
@@ -318,6 +342,22 @@ static block_t *PacketizeAVC1( decoder_t *p_dec, block_t **pp_block )
     p_block = *pp_block;
     *pp_block = NULL;
 
+#if 0
+    if( //(p_block->i_flags & BLOCK_FLAG_TYPE_I) &&
+        p_sys->p_sps && p_sys->p_pps )
+    {
+        block_t *p_pic;
+        block_t *p_sps = block_Duplicate( p_sys->p_sps );
+        block_t *p_pps = block_Duplicate( p_sys->p_pps );
+        p_sps->i_dts = p_pps->i_dts = p_block->i_dts;
+        p_sps->i_pts = p_pps->i_pts = p_block->i_pts;
+        p_pic = ParseNALBlock( p_dec, p_sps );
+        if( p_pic ) block_ChainAppend( &p_ret, p_pic );
+        p_pic = ParseNALBlock( p_dec, p_pps );
+        if( p_pic ) block_ChainAppend( &p_ret, p_pic );
+    }
+#endif
+
     for( p = p_block->p_buffer; p < &p_block->p_buffer[p_block->i_buffer]; )
     {
         block_t *p_pic;
@@ -417,6 +457,16 @@ static block_t *ParseNALBlock( decoder_t *p_dec, block_t *p_frag )
     const int i_nal_ref_idc = (p_frag->p_buffer[3] >> 5)&0x03;
     const int i_nal_type = p_frag->p_buffer[3]&0x1f;
 
+#define OUTPUT \
+    do {                                                \
+        p_pic = block_ChainGather( p_sys->p_frame );    \
+        p_pic->i_length = 0;    /* FIXME */             \
+                                                        \
+        p_sys->p_frame = NULL;                          \
+        p_sys->b_slice = VLC_FALSE;                     \
+    } while(0)
+
+
     if( p_sys->b_slice && !p_sys->b_sps )
     {
         block_ChainRelease( p_sys->p_frame );
@@ -504,23 +554,9 @@ static block_t *ParseNALBlock( decoder_t *p_dec, block_t *p_frag )
         }
         p_sys->i_nal_type = i_nal_type;
 
-        if( b_pic && p_sys->b_slice )
-        {
-            p_pic = block_ChainGather( p_sys->p_frame );
-            p_pic->i_dts = p_sys->i_dts;
-            p_pic->i_pts = p_sys->i_pts;
-            p_pic->i_length = 0;    /* FIXME */
-            p_pic->i_flags = p_sys->i_flags;
-
-            /* Reset context */
-            p_sys->p_frame = NULL;
-            p_sys->b_slice = VLC_FALSE;
-        }
+        if( b_pic && p_sys->b_slice ) OUTPUT;
 
         p_sys->b_slice = VLC_TRUE;
-        p_sys->i_flags = i_pic_flags;
-        p_sys->i_dts   = p_frag->i_dts;
-        p_sys->i_pts   = p_frag->i_pts;
 
         free( dec );
     }
@@ -531,7 +567,7 @@ static block_t *ParseNALBlock( decoder_t *p_dec, block_t *p_frag )
         bs_t s;
         int i_tmp;
 
-        msg_Dbg( p_dec, "found NAL_SPS" );
+        if( !p_sys->b_sps ) msg_Dbg( p_dec, "found NAL_SPS" );
 
         p_sys->b_sps = VLC_TRUE;
 
@@ -630,22 +666,40 @@ static block_t *ParseNALBlock( decoder_t *p_dec, block_t *p_frag )
                     w = bs_read( &s, 16 );
                     h = bs_read( &s, 16 );
                 }
-                p_dec->fmt_out.video.i_aspect =
-                    VOUT_ASPECT_FACTOR * w / h * p_dec->fmt_out.video.i_width /
-                    p_dec->fmt_out.video.i_height;
+                if( h != 0 )
+                    p_dec->fmt_out.video.i_aspect = VOUT_ASPECT_FACTOR * w /
+                        h * p_dec->fmt_out.video.i_width /
+                        p_dec->fmt_out.video.i_height;
+                else
+                    p_dec->fmt_out.video.i_aspect = VOUT_ASPECT_FACTOR;
             }
         }
 
         free( dec );
+
+
+        if( p_sys->b_slice ) OUTPUT;
     }
     else if( i_nal_type == NAL_PPS )
     {
         bs_t s;
         bs_init( &s, &p_frag->p_buffer[4], p_frag->i_buffer - 4 );
 
+        if( !p_sys->b_pps ) msg_Dbg( p_dec, "found NAL_PPS" );
+        p_sys->b_pps = VLC_TRUE;
+
         /* TODO */
-        msg_Dbg( p_dec, "found NAL_PPS" );
+
+        if( p_sys->b_slice ) OUTPUT;
     }
+    else if( i_nal_type == NAL_AU_DELIMITER ||
+             i_nal_type == NAL_SEI ||
+             ( i_nal_type >= 13 && i_nal_type <= 18 ) )
+    {
+        if( p_sys->b_slice ) OUTPUT;
+    }
+
+#undef OUTPUT
 
     /* Append the block */
     block_ChainAppend( &p_sys->p_frame, p_frag );
