@@ -1,8 +1,8 @@
 /*****************************************************************************
  * duplicate.c: duplicate stream output module
  *****************************************************************************
- * Copyright (C) 2003-2004 VideoLAN
- * $Id: duplicate.c 10101 2005-03-02 16:47:31Z robux4 $
+ * Copyright (C) 2003-2004 the VideoLAN team
+ * $Id: duplicate.c 11702 2005-07-11 09:45:54Z massiot $
  *
  * Author: Laurent Aimar <fenrir@via.ecp.fr>
  *
@@ -362,7 +362,7 @@ static vlc_bool_t ESSelected( es_format_t *fmt, char *psz_select )
         if( !strncmp( psz, "no-audio", strlen( "no-audio" ) ) ||
             !strncmp( psz, "noaudio", strlen( "noaudio" ) ) )
         {
-            if( i_cat != 1 )
+            if( i_cat == -1 )
             {
                 i_cat = fmt->i_cat != AUDIO_ES ? 1 : 0;
             }
@@ -370,7 +370,7 @@ static vlc_bool_t ESSelected( es_format_t *fmt, char *psz_select )
         else if( !strncmp( psz, "no-video", strlen( "no-video" ) ) ||
                  !strncmp( psz, "novideo", strlen( "novideo" ) ) )
         {
-            if( i_cat != 1 )
+            if( i_cat == -1 )
             {
                 i_cat = fmt->i_cat != VIDEO_ES ? 1 : 0;
             }
@@ -378,28 +378,28 @@ static vlc_bool_t ESSelected( es_format_t *fmt, char *psz_select )
         else if( !strncmp( psz, "no-spu", strlen( "no-spu" ) ) ||
                  !strncmp( psz, "nospu", strlen( "nospu" ) ) )
         {
-            if( i_cat != 1 )
+            if( i_cat == -1 )
             {
                 i_cat = fmt->i_cat != SPU_ES ? 1 : 0;
             }
         }
         else if( !strncmp( psz, "audio", strlen( "audio" ) ) )
         {
-            if( i_cat != 1 )
+            if( i_cat == -1 )
             {
                 i_cat = fmt->i_cat == AUDIO_ES ? 1 : 0;
             }
         }
         else if( !strncmp( psz, "video", strlen( "video" ) ) )
         {
-            if( i_cat != 1 )
+            if( i_cat == -1 )
             {
                 i_cat = fmt->i_cat == VIDEO_ES ? 1 : 0;
             }
         }
         else if( !strncmp( psz, "spu", strlen( "spu" ) ) )
         {
-            if( i_cat != 1 )
+            if( i_cat == -1 )
             {
                 i_cat = fmt->i_cat == SPU_ES ? 1 : 0;
             }
@@ -411,31 +411,31 @@ static vlc_bool_t ESSelected( es_format_t *fmt, char *psz_select )
 
             if( !strcmp( psz, "no-es" ) || !strcmp( psz, "noes" ) )
             {
-                if( i_es != 1 )
+                if( i_es == -1 )
                 {
-                    i_es = !NumInRange( psz_arg, fmt->i_id ) ? 1 : 0;
+                    i_es = NumInRange( psz_arg, fmt->i_id ) ? 0 : -1;
                 }
             }
             else if( !strcmp( psz, "es" ) )
             {
-                if( i_es != 1 )
+                if( i_es == -1 )
                 {
-                    i_es = NumInRange( psz_arg, fmt->i_id) ? 1 : 0;
+                    i_es = NumInRange( psz_arg, fmt->i_id) ? 1 : -1;
                 }
             }
             else if( !strcmp( psz, "no-prgm" ) || !strcmp( psz, "noprgm" ) ||
                       !strcmp( psz, "no-program" ) || !strcmp( psz, "noprogram" ) )
             {
-                if( fmt->i_group >= 0 && i_prgm != 1 )
+                if( fmt->i_group >= 0 && i_prgm == -1 )
                 {
-                    i_prgm = !NumInRange( psz_arg, fmt->i_group ) ? 1 : 0;
+                    i_prgm = NumInRange( psz_arg, fmt->i_group ) ? 0 : -1;
                 }
             }
             else if( !strcmp( psz, "prgm" ) || !strcmp( psz, "program" ) )
             {
-                if( fmt->i_group >= 0 && i_prgm != 1 )
+                if( fmt->i_group >= 0 && i_prgm == -1 )
                 {
-                    i_prgm = NumInRange( psz_arg, fmt->i_group ) ? 1 : 0;
+                    i_prgm = NumInRange( psz_arg, fmt->i_group ) ? 1 : -1;
                 }
             }
         }
@@ -449,10 +449,9 @@ static vlc_bool_t ESSelected( es_format_t *fmt, char *psz_select )
 
     free( psz_dup );
 
-    if( i_cat == 0 || i_es == 0 || i_prgm == 0 )
+    if( i_cat == 1 || i_es == 1 || i_prgm == 1 )
     {
-        /* One test failed */
-        return VLC_FALSE;
+        return VLC_TRUE;
     }
-    return VLC_TRUE;
+    return VLC_FALSE;
 }
