@@ -1,8 +1,8 @@
 /*****************************************************************************
  * i420_rgb16.c : YUV to bitmap RGB conversion module for vlc
  *****************************************************************************
- * Copyright (C) 2000 VideoLAN
- * $Id: i420_rgb16.c 8583 2004-08-30 01:24:08Z gbazin $
+ * Copyright (C) 2000 the VideoLAN team
+ * $Id: i420_rgb16.c 12684 2005-09-26 10:15:02Z sam $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -34,10 +34,14 @@
 #if defined (MODULE_NAME_IS_i420_rgb)
 #   include "i420_rgb_c.h"
 #elif defined (MODULE_NAME_IS_i420_rgb_mmx)
+#   if defined(HAVE_MMX_INTRINSICS)
+#       include <mmintrin.h>
+#   endif
 #   include "i420_rgb_mmx.h"
 #endif
 
-static void SetOffset( int, int, int, int, vlc_bool_t *, int *, int * );
+static void SetOffset( int, int, int, int, vlc_bool_t *,
+                       unsigned int *, int * );
 
 #if defined (MODULE_NAME_IS_i420_rgb)
 /*****************************************************************************
@@ -60,7 +64,7 @@ void E_(I420_RGB16_dithering)( vout_thread_t *p_vout, picture_t *p_src,
     uint8_t  *p_v   = p_src->V_PIXELS;
 
     vlc_bool_t   b_hscale;                        /* horizontal scaling type */
-    int          i_vscale;                          /* vertical scaling type */
+    unsigned int i_vscale;                          /* vertical scaling type */
     unsigned int i_x, i_y;                /* horizontal and vertical indexes */
     unsigned int i_real_y;                                          /* y % 4 */
 
@@ -278,6 +282,14 @@ void E_(I420_RGB16)( vout_thread_t *p_vout, picture_t *p_src,
             /* 15bpp 5/5/5 */
             for ( i_x = p_vout->render.i_width / 8; i_x--; )
             {
+#   if defined (HAVE_MMX_INTRINSICS)
+                __m64 mm0, mm1, mm2, mm3, mm4, mm5, mm6, mm7;
+                uint64_t tmp64;
+                INTRINSICS_INIT_16
+                INTRINSICS_YUV_MUL
+                INTRINSICS_YUV_ADD
+                INTRINSICS_UNPACK_15
+#   else
                 __asm__( MMX_INIT_16
                          : : "r" (p_y), "r" (p_u), "r" (p_v), "r" (p_buffer) );
 
@@ -286,6 +298,7 @@ void E_(I420_RGB16)( vout_thread_t *p_vout, picture_t *p_src,
                          MMX_YUV_ADD
                          MMX_UNPACK_15
                          : : "r" (p_y), "r" (p_u), "r" (p_v), "r" (p_buffer) );
+#   endif
 
                 p_y += 8;
                 p_u += 4;
@@ -298,6 +311,14 @@ void E_(I420_RGB16)( vout_thread_t *p_vout, picture_t *p_src,
             /* 16bpp 5/6/5 */
             for ( i_x = p_vout->render.i_width / 8; i_x--; )
             {
+#   if defined (HAVE_MMX_INTRINSICS)
+                __m64 mm0, mm1, mm2, mm3, mm4, mm5, mm6, mm7;
+                uint64_t tmp64;
+                INTRINSICS_INIT_16
+                INTRINSICS_YUV_MUL
+                INTRINSICS_YUV_ADD
+                INTRINSICS_UNPACK_16
+#   else
                 __asm__( MMX_INIT_16
                          : : "r" (p_y), "r" (p_u), "r" (p_v), "r" (p_buffer) );
 
@@ -306,6 +327,7 @@ void E_(I420_RGB16)( vout_thread_t *p_vout, picture_t *p_src,
                          MMX_YUV_ADD
                          MMX_UNPACK_16
                          : : "r" (p_y), "r" (p_u), "r" (p_v), "r" (p_buffer) );
+#   endif
 
                 p_y += 8;
                 p_u += 4;
@@ -319,6 +341,12 @@ void E_(I420_RGB16)( vout_thread_t *p_vout, picture_t *p_src,
          * at least we have all the pixels */
         if( i_rewind )
         {
+#if defined (MODULE_NAME_IS_i420_rgb_mmx)
+#   if defined (HAVE_MMX_INTRINSICS)
+            __m64 mm0, mm1, mm2, mm3, mm4, mm5, mm6, mm7;
+            uint64_t tmp64;
+#   endif
+#endif
             p_y -= i_rewind;
             p_u -= i_rewind >> 1;
             p_v -= i_rewind >> 1;
@@ -329,26 +357,43 @@ void E_(I420_RGB16)( vout_thread_t *p_vout, picture_t *p_src,
             CONVERT_YUV_PIXEL(2);  CONVERT_Y_PIXEL(2);
             CONVERT_YUV_PIXEL(2);  CONVERT_Y_PIXEL(2);
 #elif defined (MODULE_NAME_IS_i420_rgb_mmx)
+
+#   if defined (HAVE_MMX_INTRINSICS)
+            INTRINSICS_INIT_16
+#   else
             __asm__( MMX_INIT_16
                      : : "r" (p_y), "r" (p_u), "r" (p_v), "r" (p_buffer) );
+#   endif
 
             if( p_vout->output.i_rmask == 0x7c00 )
             {
                 /* 15bpp 5/5/5 */
+#   if defined (HAVE_MMX_INTRINSICS)
+                INTRINSICS_YUV_MUL
+                INTRINSICS_YUV_ADD
+                INTRINSICS_UNPACK_15
+#   else
                 __asm__( ".align 8"
                          MMX_YUV_MUL
                          MMX_YUV_ADD
                          MMX_UNPACK_15
                          : : "r" (p_y), "r" (p_u), "r" (p_v), "r" (p_buffer) );
+#   endif
             }
             else
             {
+#   if defined (HAVE_MMX_INTRINSICS)
+                INTRINSICS_YUV_MUL
+                INTRINSICS_YUV_ADD
+                INTRINSICS_UNPACK_16
+#   else
                 /* 16bpp 5/6/5 */
                 __asm__( ".align 8"
                          MMX_YUV_MUL
                          MMX_YUV_ADD
                          MMX_UNPACK_16
                          : : "r" (p_y), "r" (p_u), "r" (p_v), "r" (p_buffer) );
+#   endif
             }
 
             p_y += 8;
@@ -453,6 +498,14 @@ void E_(I420_RGB32)( vout_thread_t *p_vout, picture_t *p_src,
             CONVERT_YUV_PIXEL(4);  CONVERT_Y_PIXEL(4);
             CONVERT_YUV_PIXEL(4);  CONVERT_Y_PIXEL(4);
 #elif defined (MODULE_NAME_IS_i420_rgb_mmx)
+#   if defined (HAVE_MMX_INTRINSICS)
+            __m64 mm0, mm1, mm2, mm3, mm4, mm5, mm6, mm7;
+            uint64_t tmp64;
+            INTRINSICS_INIT_32
+            INTRINSICS_YUV_MUL
+            INTRINSICS_YUV_ADD
+            INTRINSICS_UNPACK_32
+#   else
             __asm__( MMX_INIT_32
                      : : "r" (p_y), "r" (p_u), "r" (p_v), "r" (p_buffer) );
 
@@ -461,6 +514,7 @@ void E_(I420_RGB32)( vout_thread_t *p_vout, picture_t *p_src,
                      MMX_YUV_ADD
                      MMX_UNPACK_32
                      : : "r" (p_y), "r" (p_u), "r" (p_v), "r" (p_buffer) );
+#   endif
 
             p_y += 8;
             p_u += 4;
@@ -473,6 +527,12 @@ void E_(I420_RGB32)( vout_thread_t *p_vout, picture_t *p_src,
          * at least we have all the pixels */
         if( i_rewind )
         {
+#if defined (MODULE_NAME_IS_i420_rgb_mmx)
+#   if defined (HAVE_MMX_INTRINSICS)
+            __m64 mm0, mm1, mm2, mm3, mm4, mm5, mm6, mm7;
+            uint64_t tmp64;
+#   endif
+#endif
             p_y -= i_rewind;
             p_u -= i_rewind >> 1;
             p_v -= i_rewind >> 1;
@@ -483,6 +543,12 @@ void E_(I420_RGB32)( vout_thread_t *p_vout, picture_t *p_src,
             CONVERT_YUV_PIXEL(4);  CONVERT_Y_PIXEL(4);
             CONVERT_YUV_PIXEL(4);  CONVERT_Y_PIXEL(4);
 #elif defined (MODULE_NAME_IS_i420_rgb_mmx)
+#   if defined (HAVE_MMX_INTRINSICS)
+            INTRINSICS_INIT_32
+            INTRINSICS_YUV_MUL
+            INTRINSICS_YUV_ADD
+            INTRINSICS_UNPACK_32
+#   else
             __asm__( MMX_INIT_32
                      : : "r" (p_y), "r" (p_u), "r" (p_v), "r" (p_buffer) );
 
@@ -491,6 +557,7 @@ void E_(I420_RGB32)( vout_thread_t *p_vout, picture_t *p_src,
                      MMX_YUV_ADD
                      MMX_UNPACK_32
                      : : "r" (p_y), "r" (p_u), "r" (p_v), "r" (p_buffer) );
+#   endif
 
             p_y += 8;
             p_u += 4;
@@ -520,7 +587,7 @@ void E_(I420_RGB32)( vout_thread_t *p_vout, picture_t *p_src,
  *****************************************************************************/
 static void SetOffset( int i_width, int i_height, int i_pic_width,
                        int i_pic_height, vlc_bool_t *pb_hscale,
-                       int *pi_vscale, int *p_offset )
+                       unsigned int *pi_vscale, int *p_offset )
 {
     int i_x;                                    /* x position in destination */
     int i_scale_count;                                     /* modulo counter */
