@@ -1,8 +1,8 @@
 /*****************************************************************************
  * qte.cpp : QT Embedded plugin for vlc
  *****************************************************************************
- * Copyright (C) 1998-2003 VideoLAN
- * $Id: qte.cpp 8551 2004-08-28 17:36:02Z gbazin $
+ * Copyright (C) 1998-2003 the VideoLAN team
+ * $Id: qte.cpp 11664 2005-07-09 06:17:09Z courmisch $
  *
  * Authors: Gerald Hansink <gerald.hansink@ordain.nl>
  *          Jean-Paul Saman <jpsaman@wxs.nl>
@@ -110,6 +110,8 @@ extern "C"
 {
 
 vlc_module_begin();
+    set_category( CAT_VIDEO );
+    set_subcategory( SUBCAT_VIDEO_VOUT );
 //    add_category_hint( N_("QT Embedded"), NULL );
 //    add_string( "qte-display", "landscape", NULL, DISPLAY_TEXT, DISPLAY_LONGTEXT);
     set_description( _("QT Embedded video output") );
