@@ -1,8 +1,8 @@
 /*****************************************************************************
  * mixer.c : audio output mixing operations
  *****************************************************************************
- * Copyright (C) 2002-2004 VideoLAN
- * $Id: mixer.c,v 1.29 2004/01/06 12:02:05 zorglub Exp $
+ * Copyright (C) 2002-2004 the VideoLAN team
+ * $Id: mixer.c 11664 2005-07-09 06:17:09Z courmisch $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -43,7 +43,7 @@
  *****************************************************************************/
 int aout_MixerNew( aout_instance_t * p_aout )
 {
-    p_aout->mixer.p_module = module_Need( p_aout, "audio mixer", NULL );
+    p_aout->mixer.p_module = module_Need( p_aout, "audio mixer", NULL, 0 );
     if ( p_aout->mixer.p_module == NULL )
     {
         msg_Err( p_aout, "no suitable aout mixer" );
@@ -261,23 +261,28 @@ static int MixBuffer( aout_instance_t * p_aout )
             {
                 p_input->p_first_byte_to_mix = p_buffer->p_buffer;
             }
-            mixer_nb_bytes = p_input->p_first_byte_to_mix
-                              - p_buffer->p_buffer;
+            mixer_nb_bytes = p_input->p_first_byte_to_mix - p_buffer->p_buffer;
 
             if ( !((i_nb_bytes + p_aout->mixer.mixer.i_bytes_per_frame
                      > mixer_nb_bytes) &&
                    (i_nb_bytes < p_aout->mixer.mixer.i_bytes_per_frame
                      + mixer_nb_bytes)) )
             {
-                msg_Warn( p_aout,
-                          "mixer start isn't output start ("I64Fd")",
+                msg_Warn( p_aout, "mixer start isn't output start ("I64Fd")",
                           i_nb_bytes - mixer_nb_bytes );
 
                 /* Round to the nearest multiple */
                 i_nb_bytes /= p_aout->mixer.mixer.i_bytes_per_frame;
                 i_nb_bytes *= p_aout->mixer.mixer.i_bytes_per_frame;
-                p_input->p_first_byte_to_mix = p_buffer->p_buffer
-                                                + i_nb_bytes;
+                if( i_nb_bytes < 0 )
+                {
+                    /* Is it really the best way to do it ? */
+                    aout_FifoSet( p_aout, &p_aout->output.fifo, 0 );
+                    aout_DateSet( &exact_start_date, 0 );
+                    break;
+                }
+
+                p_input->p_first_byte_to_mix = p_buffer->p_buffer + i_nb_bytes;
             }
         }
     }
