@@ -1,8 +1,8 @@
 /*****************************************************************************
  * iso_lang.c: function to decode language code (in dvd or a52 for instance).
  *****************************************************************************
- * Copyright (C) 1998-2001 VideoLAN
- * $Id: iso_lang.c,v 1.10 2003/12/22 14:23:14 sam Exp $
+ * Copyright (C) 1998-2004 the VideoLAN team
+ * $Id: iso_lang.c 11716 2005-07-12 18:01:14Z courmisch $
  *
  * Author: Stphane Borel <stef@via.ecp.fr>
  *         Arnaud de Bossoreille de Ribou <bozo@via.ecp.fr>
@@ -54,18 +54,20 @@ const char * DecodeLanguage( uint16_t i_code )
 
     for( p_lang = p_languages; p_lang->psz_eng_name; p_lang++ )
     {
-        if( !strncmp( p_lang->psz_iso639_1, psz_code, 2 ) )
+        if( !memcmp( p_lang->psz_iso639_1, psz_code, 2 ) )
         {
+# if 0
             if( *p_lang->psz_native_name )
             {
                 return p_lang->psz_native_name;
             }
+#endif
 
-            return p_lang->psz_eng_name;
+            return _( p_lang->psz_eng_name );
         }
     }
 
-    return "Unknown";
+    return _( "Unknown" );
 }
 
 const iso639_lang_t * GetLang_1( const char * psz_code )
