@@ -1,8 +1,8 @@
 /*****************************************************************************
  * threads.c : threads implementation for the VideoLAN client
  *****************************************************************************
- * Copyright (C) 1999-2004 VideoLAN
- * $Id: threads.c 10706 2005-04-16 12:30:45Z courmisch $
+ * Copyright (C) 1999-2004 the VideoLAN team
+ * $Id: threads.c 12854 2005-10-16 18:01:09Z hartman $
  *
  * Authors: Jean-Marc Dressler <polux@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
@@ -343,7 +343,7 @@ int __vlc_mutex_destroy( char * psz_file, int i_line, vlc_mutex_t *p_mutex )
 
 #elif defined( PTHREAD_COND_T_IN_PTHREAD_H )
     i_result = pthread_mutex_destroy( &p_mutex->mutex );
-    if ( i_result )
+    if( i_result )
     {
         i_thread = (int)pthread_self();
         psz_error = strerror(i_result);
@@ -483,13 +483,16 @@ int __vlc_cond_destroy( char * psz_file, int i_line, vlc_cond_t *p_condvar )
         i_result = !CloseHandle( p_condvar->event )
           || !CloseHandle( p_condvar->semaphore );
 
+    if( p_condvar->semaphore != NULL )
+		DeleteCriticalSection( &p_condvar->csection );
+
 #elif defined( HAVE_KERNEL_SCHEDULER_H )
     p_condvar->init = 0;
     return 0;
 
 #elif defined( PTHREAD_COND_T_IN_PTHREAD_H )
     i_result = pthread_cond_destroy( &p_condvar->cond );
-    if ( i_result )
+    if( i_result )
     {
         i_thread = (int)pthread_self();
         psz_error = strerror(i_result);
@@ -549,9 +552,9 @@ int __vlc_thread_create( vlc_object_t *p_this, char * psz_file, int i_line,
 #endif
     }
 
-    if ( p_this->thread_id && i_priority )
+    if( p_this->thread_id && i_priority )
     {
-        if ( !SetThreadPriority(p_this->thread_id, i_priority) )
+        if( !SetThreadPriority(p_this->thread_id, i_priority) )
         {
             msg_Warn( p_this, "couldn't set a faster priority" );
             i_priority = 0;
@@ -569,15 +572,18 @@ int __vlc_thread_create( vlc_object_t *p_this, char * psz_file, int i_line,
     i_ret = pthread_create( &p_this->thread_id, NULL, func, p_data );
 
 #ifndef SYS_DARWIN
-    if ( config_GetInt( p_this, "rt-priority" ) )
+    if( config_GetInt( p_this, "rt-priority" ) )
 #endif
     {
         int i_error, i_policy;
         struct sched_param param;
 
         memset( &param, 0, sizeof(struct sched_param) );
-        i_priority += config_GetInt( p_this, "rt-offset" );
-        if ( i_priority <= 0 )
+        if( config_GetType( p_this, "rt-offset" ) )
+        {
+            i_priority += config_GetInt( p_this, "rt-offset" );
+        }
+        if( i_priority <= 0 )
         {
             param.sched_priority = (-1) * i_priority;
             i_policy = SCHED_OTHER;
@@ -587,7 +593,7 @@ int __vlc_thread_create( vlc_object_t *p_this, char * psz_file, int i_line,
             param.sched_priority = i_priority;
             i_policy = SCHED_RR;
         }
-        if ( (i_error = pthread_setschedparam( p_this->thread_id,
+        if( (i_error = pthread_setschedparam( p_this->thread_id,
                                                i_policy, &param )) )
         {
             msg_Warn( p_this, "couldn't set thread priority (%s:%d): %s",
@@ -648,7 +654,7 @@ int __vlc_thread_set_priority( vlc_object_t *p_this, char * psz_file,
 {
 #if defined( PTH_INIT_IN_PTH_H ) || defined( ST_INIT_IN_ST_H )
 #elif defined( WIN32 ) || defined( UNDER_CE )
-    if ( !SetThreadPriority(GetCurrentThread(), i_priority) )
+    if( !SetThreadPriority(GetCurrentThread(), i_priority) )
     {
         msg_Warn( p_this, "couldn't set a faster priority" );
         return 1;
@@ -656,15 +662,18 @@ int __vlc_thread_set_priority( vlc_object_t *p_this, char * psz_file,
 
 #elif defined( PTHREAD_COND_T_IN_PTHREAD_H )
 #ifndef SYS_DARWIN
-    if ( config_GetInt( p_this, "rt-priority" ) )
+    if( config_GetInt( p_this, "rt-priority" ) )
 #endif
     {
         int i_error, i_policy;
         struct sched_param param;
 
         memset( &param, 0, sizeof(struct sched_param) );
-        i_priority += config_GetInt( p_this, "rt-offset" );
-        if ( i_priority <= 0 )
+        if( config_GetType( p_this, "rt-offset" ) )
+        {
+            i_priority += config_GetInt( p_this, "rt-offset" );
+        }
+        if( i_priority <= 0 )
         {
             param.sched_priority = (-1) * i_priority;
             i_policy = SCHED_OTHER;
@@ -674,9 +683,9 @@ int __vlc_thread_set_priority( vlc_object_t *p_this, char * psz_file,
             param.sched_priority = i_priority;
             i_policy = SCHED_RR;
         }
-        if ( !p_this->thread_id )
+        if( !p_this->thread_id )
             p_this->thread_id = pthread_self();
-        if ( (i_error = pthread_setschedparam( p_this->thread_id,
+        if( (i_error = pthread_setschedparam( p_this->thread_id,
                                                i_policy, &param )) )
         {
             msg_Warn( p_this, "couldn't set thread priority (%s:%d): %s",
